#!BPY

"""
Name: 'FPlot'
Blender: 242
Group: 'Add'
Tooltip: 'Add a function plot.'
"""


__author__ = "Rebecca Breu"
__url__ = ("blender", "elysiun")
__version__ = "31.01.2007"

__bpydoc__ = """Dieses Script erstellt einen Plot einer 2D-Funktion.
Der User kann die Funktion und den Funktionsbereich ueber eine GUI
auswaehlen."""

import Blender
from Blender import NMesh
from Blender.Draw import *
from math import *


class Function:
    """Klasse, die eine mathematische Funktion repraesentiert."""
    
    def __init__(self, func):
        self.func = func # Die Funktionsvorschritf als String
        
        self.xstep = 0.1 # Schrittweite in x-Richtung
        self.xstart = 0.0 # Definitionsbereich fuer x
        self.xstop = 1.0  # dito

        self.ystep = 0.1 # Schrittweite in y-Richtung
        self.ystart = 0.0 # Definitionsbereich fuer y
        self.ystop = 1.0 # dito
        

    def plot(self):
        """
        Hier wird die Funktion geplottet, d.h. ein Mesh-Objekt erstellt.
        Die Funktionsvorschrift wird mit eval ausgewertet
        """

        # Berechne die Gitterpunkte, an denen die Funktion ausgewertet
        # werden soll (abhaengig von Schrittweite und Definitionsbereich):
        xvals = [self.xstep * i for i in range(int(self.xstart/self.xstep),
                                               int(self.xstop/self.xstep))]
        yvals = [self.ystep * i for i in range(int(self.ystart/self.ystep),
                                               int(self.ystop/self.ystep))]
        
        mesh = NMesh.New()

        # Berechne die Punkte des Meshs:
        for x in xvals:
            for y in yvals:
                # Damit der Benutzer neben x und y auch den Radius in der
                # Funktionsvorschrit benutzen kann:
                r = sqrt(x**2 + y**2)

                # Fuege dem Mesh einen Funktionsplot-Punkt hinzu:
                mesh.verts.append(NMesh.Vert(x, y, eval(self.func)))

                
        xnum = len(xvals)
        ynum = len(yvals)

        # Und jezt bekommt das Mesh noch Flaechen:
        for i in range(xnum-1):
            for j in range(ynum-1):
                flaeche = NMesh.Face()
                flaeche.v.append(mesh.verts[i * ynum + j])
                flaeche.v.append(mesh.verts[(i + 1) * ynum + j])
                flaeche.v.append(mesh.verts[(i + 1) * ynum + j + 1])
                flaeche.v.append(mesh.verts[i * ynum + j + 1])
                mesh.faces.append(flaeche)

        # Das Plot-Mes ist fertig, fuege es hinzu:
        obj = Blender.Object.New("Mesh", "Plot")
        obj.link(mesh)

        current_scene = Blender.Scene.GetCurrent()
        current_scene.link(obj)
        Blender.Redraw()



class GUI:
    """Zeichnet die GUI und stellt Event-Handler bereit."""

    def __init__(self):
        # ID's fuer unsere Events:
        self.QUIT_BUTTON = 1
        self.PLOT_BUTTON = 2
        self.VAL_BUTTONS = 3

        self.func = Function("sin(x) * sin(y)")
                

    def draw(self):
        """Zeichne die GUI."""
        Button("Plot", self.PLOT_BUTTON, 10, 10, 60, 40)
        Button("Quit", self.QUIT_BUTTON, 80, 10, 60, 40)
        self.fstring = String("Function:", self.VAL_BUTTONS, 10, 150, 320, 20,
                              self.func.func, 200,
                              "Function to be plotted. Use x, y and r as parameters")
        self.xmin = Slider("xmin", self.VAL_BUTTONS, 10, 110, 150, 20,
                           self.func.xstart, -100.0, 100.0, 0,
                           "Domain of definition")
        self.xmax = Slider("xmax", self.VAL_BUTTONS, 10, 90, 150, 20,
                           self.func.xstop, -100.0, 100.0, 0,
                           "Domain of definition")
        self.xstep = Slider("xstep", self.VAL_BUTTONS, 10, 70, 150, 20,
                           self.func.xstep, 0.001, 10.0, 0,
                           "Step size")
        self.ymin = Slider("ymin", self.VAL_BUTTONS, 180, 110, 150, 20,
                          self.func.ystart, -100.0, 100.0, 0,
                          "Domain of definition")
        self.ymax = Slider("ymax", self.VAL_BUTTONS, 180, 90, 150, 20,
                           self.func.ystop, -100.0, 100.0, 0,
                           "Domain of definition")
        self.ystep = Slider("ystep", self.VAL_BUTTONS, 180, 70, 150, 20,
                           self.func.ystep, 0.001, 10.0, 0,
                           "Step size")


    def event(self, evt, val):
        """Allgemeiner Event-Handler"""
        
        pass
    

    def bevent(self, evt):
        """Event-Handler fuer Buttons"""
        
        if evt == self.QUIT_BUTTON:
            Exit()

        if evt == self.VAL_BUTTONS:
            self.func.func = self.fstring.val            
            self.func.xstart = self.xmin.val
            self.func.xstop = self.xmax.val
            self.func.xstep = self.xstep.val
            self.func.ystart = self.ymin.val
            self.func.ystop = self.ymax.val
            self.func.ystep = self.ystep.val

        if evt == self.PLOT_BUTTON:
            """Plotte die Funktion."""
            self.func.plot()


gui = GUI()
Register(gui.draw, gui.event, gui.bevent)
